<?php

/**
 * The admin-specific functionality of the plugin.
 *
 * @link       https://inbank.ee
 * @since      1.0.0
 *
 * @package    Woocommerce_Inbank
 * @subpackage Woocommerce_Inbank/admin
 */

/**
 * The admin-specific functionality of the plugin.
 *
 * Defines the plugin name, version, and two examples hooks for how to
 * enqueue the admin-specific stylesheet and JavaScript.
 *
 * @package    Woocommerce_Inbank
 * @subpackage Woocommerce_Inbank/admin
 * @author     Inbank
 */
class Woocommerce_Inbank_Admin {

	/**
	 * The ID of this plugin.
	 *
	 * @since    1.0.0
	 * @access   private
	 * @var      string    $woocommerce_inbank    The ID of this plugin.
	 */
	private $woocommerce_inbank;

	/**
	 * The version of this plugin.
	 *
	 * @since    1.0.0
	 * @access   private
	 * @var      string    $version    The current version of this plugin.
	 */
	private $version;

	/**
	 * Is WCML plugin active
	 *
	 * @since    1.0.0
	 * @access   private
	 * @var      string     $is_wcml_plugin_active   is plugin active
	 */
	private $is_wcml_plugin_active;

	/**
	 * Initialize the class and set its properties.
	 *
	 * @since    1.0.0
	 * @param      string    $woocommerce_inbank       The name of this plugin.
	 * @param      string    $version    The version of this plugin.
	 */
	public function __construct( $woocommerce_inbank, $version ) {

		$this->woocommerce_inbank = $woocommerce_inbank;
		$this->version = $version;

        $this->load_dependencies();
	}

    /**
     * Load the required dependencies for the Admin facing functionality.
     *
     * Include the following files that make up the plugin:
     *
     * - Wppb_Demo_Plugin_Admin_Settings. Registers the admin settings and page.
     *
     *
     * @since    1.0.0
     * @access   private
     */
    private function load_dependencies()
    {
        require_once plugin_dir_path( dirname( __FILE__ ) ) .  'admin/class-woocommerce-inbank-settings.php';
        require_once plugin_dir_path( dirname( __FILE__ ) ) .  'admin/class-woocommerce-inbank-payment-list-table.php';
    }

	/**
	 * Register the stylesheets for the admin area.
	 *
	 * @since    1.0.0
	 */
	public function enqueue_styles() {

		/**
		 * This function is provided for demonstration purposes only.
		 *
		 * An instance of this class should be passed to the run() function
		 * defined in Woocommerce_Inbank_Loader as all of the hooks are defined
		 * in that particular class.
		 *
		 * The Woocommerce_Inbank_Loader will then create the relationship
		 * between the defined hooks and the functions defined in this
		 * class.
		 */
		wp_enqueue_style( $this->woocommerce_inbank, plugin_dir_url( __FILE__ ) . 'css/woocommerce-inbank-admin.css', array(), $this->version, 'all' );
        wp_enqueue_style('select2', 'https://cdnjs.cloudflare.com/ajax/libs/select2/4.0.3/css/select2.min.css' );

	}

	/**
	 * Register the JavaScript for the admin area.
	 *
	 * @since    1.0.0
	 */
	public function enqueue_scripts() {

		/**
		 * This function is provided for demonstration purposes only.
		 *
		 * An instance of this class should be passed to the run() function
		 * defined in Woocommerce_Inbank_Loader as all of the hooks are defined
		 * in that particular class.
		 *
		 * The Woocommerce_Inbank_Loader will then create the relationship
		 * between the defined hooks and the functions defined in this
		 * class.
		 */

        wp_enqueue_script('select2', 'https://cdnjs.cloudflare.com/ajax/libs/select2/4.0.3/js/select2.min.js', array('jquery') );
		wp_enqueue_script( $this->woocommerce_inbank, plugin_dir_url( __FILE__ ) . 'js/woocommerce-inbank-admin.js', array( 'jquery' ), $this->version, true );

	}

    /**
     * Method used for product search ajax request
     *
     * @return void
     */
    public function ajax_get_products() {
        $return = array();

        $exclude_ids = '';
        if (isset($_GET['exclude_ids']))
        {
            $exclude_ids = $_GET['exclude_ids'];
            $exclude_ids = array_map('intval', explode(',', $exclude_ids));
        }

        // you can use WP_Query, query_posts() or get_posts() here - it doesn't matter
        $search_results = new WP_Query( array(
            's'=> $_GET['q'], // the search query
            'post_status' => 'publish', // if you don't want drafts to be returned
            'ignore_sticky_posts' => 1,
            'posts_per_page' => 20,
            'post_type' => 'product',
            'post__not_in' => !empty($exclude_ids) ? $exclude_ids : false
        ) );

        $no_image_url = wc_placeholder_img_src( 'woocommerce_thumbnail' );


        if( $search_results->have_posts() ) :
            while( $search_results->have_posts() ) : $search_results->the_post();
                // shorten the title a little
                $title = ( mb_strlen( $search_results->post->post_title ) > 50 ) ? mb_substr( $search_results->post->post_title, 0, 49 ) . '...' : $search_results->post->post_title;
                $image = get_the_post_thumbnail_url( $search_results->post->ID );
                $return[] = array( 'id_product' => $search_results->post->ID, 'title' => $title, 'img_url' => (empty($image) ? $no_image_url : $image) ); // array( Post ID, Post Title )
            endwhile;
        endif;
        echo json_encode( $return );
        die;
    }

    /**
     * Fix inbank payment name in woocommerce order list page
     *
     * @param $title
     * @param $object
     */
    public function fix_inbank_payment_name_in_orders_list( $title, $object )
    {
        if ($this->is_wcml_plugin_active === null && function_exists('is_plugin_active'))
            $this->is_wcml_plugin_active = is_plugin_active( 'woocommerce-multilingual/wpml-woocommerce.php' );

        if (
            $this->is_wcml_plugin_active &&
            method_exists($object, 'get_payment_method') &&
            strpos($object->get_payment_method(), 'inbank_') !== false)
        {
            $order_data = $object->get_data();
            $order_changes = $object->get_changes();
            $prop = 'payment_method_title';

            if ( array_key_exists( $prop, $order_data ) ) {
                $title = array_key_exists( $prop, $order_changes ) ? $order_changes[ $prop ] : $order_data[ $prop ];
            }
        }

        return $title;
    }

    /**
     * Fix inbank payment name in woocommerce order list page
     *
     * @param $title
     * @param $object
     */
    public function fix_inbank_payment_name_in_order_page( $title, $payment_id )
    {
        global $pagenow;

        if (is_admin() && $pagenow == 'post.php' && strpos($payment_id, 'inbank_') !== false)
        {
            global $post;
            $order_id = $post->ID;
            if ($post->post_type == 'shop_order')
            {
                $order = new WC_Order($order_id);
                $order_changes = $order->get_changes();
                $order_data = $order->get_data();
                $prop = 'payment_method_title';

                if (array_key_exists($prop, $order_data)) {
                    $title = array_key_exists($prop, $order_changes) ? $order_changes[$prop] : $order_data[$prop];
                }
            }
        }

        return $title;
    }


    function media_selector_print_scripts() {

        ?><script type='text/javascript'>

            jQuery( document ).ready( function( $ ) {

                // Uploading files
                var file_frame;

                jQuery('#upload_image_button').on('click', function( event ){

                    event.preventDefault();

                    // If the media frame already exists, reopen it.
                    if ( file_frame ) {
                        // Open frame
                        file_frame.open();
                        return;
                    }

                    // Create the media frame.
                    file_frame = wp.media.frames.file_frame = wp.media({
                        title: 'Select a image to upload',
                        button: {
                            text: 'Use this image',
                        },
                        multiple: false	// Set to true to allow multiple files to be selected
                    });

                    // When an image is selected, run a callback.
                    file_frame.on( 'select', function() {
                        // We set multiple to false so only get one image from the uploader
                        attachment = file_frame.state().get('selection').first().toJSON();

                        // Do something with attachment.id and/or attachment.url here
                        $( '.image-preview-wrapper' ).css( 'display', '' );
                        $( '#delete_image_button' ).css( 'display', '' );
                        $( '#image-preview' ).attr( 'src', attachment.url ).css( 'width', 'auto' );
                        $( '#image_attachment_id' ).val( attachment.id );
                    });

                    file_frame.on('open', function(){
                        var selection = file_frame.state().get('selection');
                        var selected = $('#image_attachment_id').val(); // the id of the image
                        if (selected) {
                            selection.add(wp.media.attachment(selected));
                        }
                    });

                    // Finally, open the modal
                    file_frame.open();
                });

                $('#delete_image_button').on('click', function (e) {
                    $( '#delete_image_button' ).css( 'display', 'none' );
                    $( '#image_attachment_id' ).val( '' );
                    $( '.image-preview-wrapper' ).css( 'display', 'none' );
                });
            });

        </script><?php

    }
}
