<?php



class Woocommerce_Inbank_Settings {

    private $plugin_name;
    private $version;


    public function __construct( $plugin_name, $version ) {

        $this->plugin_name = $plugin_name;
        $this->version = $version;
    }


    public function setup_plugin_options_menu() {

        //Add the menu to the Plugins set of menu items
        add_options_page(
            'Inbank settings', 					// The title to be displayed in the browser window for this page.
            'Inbank',					// The text to be displayed for this menu item
            'manage_options',					// Which type of users can see this menu item
            'woocommerce-inbank-settings',			// The unique ID - that is, the slug - for this menu item
            array( $this, 'render_settings_page_content')				// The name of the function to call when rendering this menu's page
        );

    }


    public function filter_add_action_links($links) {
        $settings_link = [
            '<a href="' . admin_url( 'options-general.php?page=woocommerce-inbank-settings') . '">' . __('Settings', 'woocommerce-inbank') . '</a>',
        ];
        return array_merge($settings_link, $links);
    }

    /**
     * Entry point for plugins settings page.
     */
    public function render_settings_page_content()
    {
        echo "<div class=\"wrap\"><h1></h1>";
        $check_prerequisites_result = InbankTools::check_prerequisites();
        if($check_prerequisites_result !== true)
        {
            echo "<div class=\"notice notice-error\"> 
                <p><strong>{$check_prerequisites_result}</strong></p>
            </div>";

            return;
        }

        $this->postProcess();

        $this->render_messages();
        $this->render_header();

        $action = InbankTools::getValue('action', '');
        switch($action) {
            case 'add_payment':
                $this->render_add_payment_form();
                break;
            case 'edit_payment':
                $this->render_step_content();
                break;
            case 'enable_disable_payment':
                $this->enable_disable_payment();
                break;
            case 'delete_payment':
                $this->delete_payment();
                break;
            case 'update_products':
                $this->update_products();
                break;
            case 'force_update_db':
                $this->force_update_db();
                break;
            default:
                $this->render_main_page();
                break;
        }
        echo "</div>";
    }

    private function postProcess()
    {
        $step = (int)InbankTools::getValue('inbank_step', 1);
        $action = InbankTools::getValue('action', '');
        $post_data = InbankTools::getValue('woocommerce-inbank_payment', []);
        $last_step = 2;


        if (!empty($_POST))
        {
            if ($action == 'add_payment' && $step == 1)
            {
                // validating first step
                $result = $this->isFirstStepDataValid($post_data);
                if ($result !== true) {
                    foreach ($result as $error)
                        InbankPaymentFormHelper::displayWarning($error);
                } else {
                    $post_data = InbankTools::getValue('woocommerce-inbank_payment', []);
                    $class_name = $post_data['class_name'];
                    if (class_exists($class_name))
                    {
                        $code = empty($post_data['product_code']) ? $post_data['product_code_debug'] : $post_data['product_code'];
                        $exists = InbankPayment::checkForDuplicate($code, $post_data['country'], $class_name);

                        if ($exists)
                        {
                            // payment exists msg
                            $error = InbankLocalization::getString(15);
                            InbankPaymentFormHelper::displayWarning($error);
                            return false;
                        }

                        //creating a dummy payment to populate inbank product with data
                        $payment = new InbankPayment();
                        $payment->hydrate($post_data);

                        $product = new $class_name($payment);
                        $product_info = $product->api->getInfo();

                        if (isset($product_info->message)) {
                            $error = implode(',', $product_info->message);
                            if (strpos($error, 'product_not_found') !== false) {
                                $error = InbankLocalization::getString(6);
                            } elseif (strpos($error, 'product_code_is_missing') !== false) {
                                $error = InbankLocalization::getString(7);
                            }

                            InbankPaymentFormHelper::displayWarning($error);
                            return false;
                        } elseif (isset($product_info->error)) {
                            InbankPaymentFormHelper::displayWarning(InbankLocalization::getString(8) . ' ' . implode(',', $product_info->error));
                            return false;
                        } else if (isset($product_info->subtype) && $product_info->subtype !== $product->subtype) {
                            InbankPaymentFormHelper::displayWarning(InbankLocalization::getString(9));
                            return false;
                        } else if (empty($product_info)) {
                            InbankPaymentFormHelper::displayWarning(InbankLocalization::getString(8));
                            return false;
                        }

                        if (!empty($product_info)) {
                            $payment->title = InbankTools::multiLang($product_info->name);
                            $payment->additional_details = InbankTools::multiLang('');
                            $payment->minimum_order_amount = $product_info->parameters->min_amount;
                            $payment->maximum_order_amount = $product_info->parameters->max_amount;
                            $payment->period = implode(",", $product_info->parameters->period_options);
                            $payment->default_period = $product_info->parameters->default_period;
                            $payment->max_period = max($product_info->parameters->period_options);
                            $payment->default_payment_day = !empty($product_info->parameters->default_payment_day) ? $product_info->parameters->default_payment_day : 10;
                            $payment->needs_approval = (int)(isset($product_info->parameters->contract_activation_requires_partner_approval) ? $product_info->parameters->contract_activation_requires_partner_approval : 0);
                            $payment->down_payment = (int)(isset($product_info->down_payment_enabled) ? $product_info->down_payment_enabled : 0);
                            $payment->api_last_upd = date('Y-m-d H:i:s');

                            if (!empty($payment->localization_id) && !empty($payment->localization_token)) {
                                PhraseApp::fillSinglePaymentTranslations($payment);
                            }

                            $payment->custom_title = $payment->title;
                            $payment->custom_additional_details = $payment->additional_details;

                            $product_data = $payment->toArray();
                            $_POST['woocommerce-inbank_payment'] = $product_data;
                            $_POST['inbank_step'] = ++$step;
                        }
                    }
                }
            }
            else if (($action == 'add_payment' && $step == $last_step) || $action == 'edit_payment')
            {
                // saving payment method
                $id_payment = (int)InbankTools::getValue('id_payment', 0);
                if ($id_payment > 0)
                    $payment = new InbankPayment($id_payment);
                else
                    $payment = new InbankPayment();

                $payment->hydrate($post_data);
                $result = $payment->save();

                if ($result !== true) {
                    // failed to save payment method
                    if (!empty($result)) {
                        foreach ($result as $error)
                            InbankPaymentFormHelper::displayWarning($error);
                    }
                    else {
                        InbankPaymentFormHelper::displayWarning(InbankLocalization::getString(5));
                    }

                } else {
                    InbankPayment::getProductInfo($payment);

                    // success & returning to main list
                    if ($action == 'add_payment')
                        $id_string = 3;
                    else
                        $id_string = 4;

                    // resetting action to get main list page
                    wp_redirect( admin_url( 'options-general.php?page=woocommerce-inbank-settings&conf='.$id_string ) );
                }
            }
            else if ($action == '' && !empty($global_settings = InbankTools::getValue('woocommerce-inbank_global_settings')))
            {
                update_option( 'woocommerce-inbank_global_settings', $global_settings );
                InbankPaymentFormHelper::displayConfirmation(InbankLocalization::getString(16));
            }
        }

        $config_file_dir = INBANK_MODULE_DIR.'configuration.json';
        if (is_file($config_file_dir)) {
            unlink($config_file_dir);
            if (is_file($config_file_dir)) {
                InbankPaymentFormHelper::displayWarning(InbankLocalization::getString(18));
            }
        }
    }

    /**
     * Renders header Inbank logo
     */
    public function render_header()
    {
        ?>
        <div class="container-fluid">
            <div class="inbank__settings">
                <div class="inbank__settings-content">
                    <div class="inbank__settings-logo inbank__settings-logo--middle">
                        <img src="<?=plugin_dir_url( __FILE__ )."../assets/images/admin_logo_inbank.svg";?>">
                    </div>
                </div>
            </div>
        </div>
        <?php
    }

    /**
     * Renders confirmation or worning message returned through URL.
     */
    public function render_messages()
    {
        $conf = (int)InbankTools::getValue('conf', 0);
        $warn = (int)InbankTools::getValue('warn', 0);

        if ($conf > 0)
            InbankPaymentFormHelper::displayConfirmation(InbankLocalization::getString($conf));

        if ($warn > 0)
            InbankPaymentFormHelper::displayWarning(InbankLocalization::getString($warn));
    }

    /**
     * Renders main page.
     */
    public function render_main_page()
    {
        $this->render_payments_list();
        $this->render_global_settings_form();
    }

    /**
     * Initializes and renders payment list table.
     */
    public function render_payments_list()
    {
        $payments_list = new Woocommerce_Inbank_Payment_List();
        $button_name = __('Add payment option', 'woocommerce-inbank');

        ?>
        <div class="wrap">
            <div id="poststuff">
                <div id="post-body" class="metabox-holder">
                    <div id="post-body-content">
                        <div class="inbank-payment-list meta-box-sortables ui-sortable">
                            <form method="post">
                                <?php
                                $payments_list->prepare_items();
                                $payments_list->display(); ?>
                            </form>
                        </div>
                    </div>
                </div>
                <br class="clear">
                <div class="inbank-settings-btn-container">
                    <?php
                    echo '<a class="inbank-settings-btn" href="'.admin_url( 'options-general.php?page=woocommerce-inbank-settings&action=add_payment').'">'.$button_name.'</a>';
                    ?>
                </div>
            </div>
        </div>
        <?php
    }

    /**
     * Initializes and renders global settings form.
     */
    public function render_global_settings_form()
    {
        echo '<div class="wrap general-settings-block"><form method="post" class="inbank-state-form" action="options-general.php?page=woocommerce-inbank-settings">';
        $this->init_global_settings_form();
        do_settings_sections( 'woocommerce-inbank_global_settings' );
        echo '<div class="submit">
                <a class="inbank-settings-btn secondary" href="'.admin_url( 'options-general.php?page=woocommerce-inbank-settings&action=update_products').'">'.__('Update products', 'woocommerce-inbank').'</a>
               <input type="submit" name="submit" id="submit" class="inbank-settings-btn" value="'.__('Save changes', 'woocommerce-inbank').'">                     
            </div>';
        echo '</form></div>';
    }

    /**
     * Global settings form initialization.
     */
    public function init_global_settings_form()
    {
        // Order states
        if (function_exists('wc_get_order_statuses'))
            $wc_order_states = call_user_func('wc_get_order_statuses');
        else
            $wc_order_states = array();

        // MD5 hash used as a token for cron url. Secured by AUTH salt which is unique for every WP site.
        $hash = md5('woocommerce-inbank'.AUTH_SALT);

        // Options
        $options = get_option( 'woocommerce-inbank_global_settings' );
        if($options == false)
        {
            $default_array = [
                'order_state_confirmed'	=>	'wc-processing',
                'order_state_canceled'	=>	'wc-cancelled',
                'order_state_declined'	=>	'wc-cancelled',
                'order_state_granted'	=>	'wc-on-hold',
                'order_state_pending'	=>	'wc-pending',
            ];

            add_option( 'woocommerce-inbank_global_settings', $default_array );
            $options = get_option( 'woocommerce-inbank_global_settings' );
        }

        add_settings_section(
            'global_settings_section',	// ID used to identify this section and with which to register options
            null,	// Title to be displayed on the administration page
            function() { $this->render_custom_form_title('global_settings_section'); },	// Callback used to render the description of the section
            'woocommerce-inbank_global_settings'		// Page on which to add this section of options
        );

        add_settings_field(
            'global_order_state_confirmed',
            __('Confirmed order state', 'woocommerce-inbank'),
            'InbankPaymentFormHelper::input_select_callback',
            'woocommerce-inbank_global_settings',
            'global_settings_section',
            array(
                'global_settings',
                'order_state_confirmed',
                $options['order_state_confirmed'],
                '',
                $wc_order_states
            )
        );

        add_settings_field(
            'global_order_state_granted',
            __('Granted order state', 'woocommerce-inbank'),
            'InbankPaymentFormHelper::input_select_callback',
            'woocommerce-inbank_global_settings',
            'global_settings_section',
            array(
                'global_settings',
                'order_state_granted',
                $options['order_state_granted'],
                '',
                $wc_order_states
            )
        );

        add_settings_field(
            'global_order_state_pending',
            __('Pending order state', 'woocommerce-inbank'),
            'InbankPaymentFormHelper::input_select_callback',
            'woocommerce-inbank_global_settings',
            'global_settings_section',
            array(
                'global_settings',
                'order_state_pending',
                $options['order_state_pending'],
                '',
                $wc_order_states
            )
        );

        add_settings_field(
            'global_order_state_canceled',
            __('Canceled order state', 'woocommerce-inbank'),
            'InbankPaymentFormHelper::input_select_callback',
            'woocommerce-inbank_global_settings',
            'global_settings_section',
            array(
                'global_settings',
                'order_state_canceled',
                isset($options['order_state_canceled']) ? $options['order_state_canceled'] : 'wc-cancelled',
                '',
                $wc_order_states
            )
        );

        add_settings_field(
            'global_order_state_declined',
            __('Declined order state', 'woocommerce-inbank'),
            'InbankPaymentFormHelper::input_select_callback',
            'woocommerce-inbank_global_settings',
            'global_settings_section',
            array(
                'global_settings',
                'order_state_declined',
                isset($options['order_state_declined']) ? $options['order_state_declined'] : 'wc-cancelled',
                '',
                $wc_order_states
            )
        );

        add_settings_field(
            'global_cron_url',
            __('Cron link', 'woocommerce-inbank'),
            'InbankPaymentFormHelper::input_url_callback',
            'woocommerce-inbank_global_settings',
            'global_settings_section',
            array(
                get_admin_url().'admin-ajax.php?action=woocommerce_inbank_cron_update&token='.$hash,
                __('By default, products are updated automatically every 24 hours. Use this cron job if you\'d like to manually update products at your chosen time.', 'woocommerce-inbank'),
            )
        );

        add_settings_field(
            'global_force_db_updates',
            __('', 'woocommerce-inbank'),
            'InbankPaymentFormHelper::input_url_callback',
            'woocommerce-inbank_global_settings',
            'global_settings_section',
            array(
                admin_url( 'options-general.php?page=woocommerce-inbank-settings&action=force_update_db'),
                '',
                __('Force DB updates in case they were not applied.', 'woocommerce-inbank')
            )
        );

        register_setting(
            'woocommerce-inbank_global_settings',
            'woocommerce-inbank_global_settings'
        );
    }

    /**
     * Renders add new payment form with steps based header.
     */
    public function render_add_payment_form()
    {
        $step = InbankTools::getValue('inbank_step', 1);

        echo '<div class="inbank-circle-container">
                <div class="inbank-circle-item '.($step == 1 ? 'active' : 'passed').'">
                    <div class="inbank-circle">
                    1
                    </div>
                    <p>Select Payment</p>
                </div>
                <div class="inbank-circle-item '.($step == 2 ? 'active' : 'passed').'">
                    <div class="inbank-circle">
                    2
                    </div>
                    <p>Payment settings</p>
                </div>
            </div>';

        $this->render_step_content();
    }

    /**
     * Renders step content for ADD and EDIT payment action.
     */
    public function render_step_content()
    {
        echo '<div class="wrap"><div class="card inbank-add-payment-form">';
        $step = InbankTools::getValue('inbank_step', 1);
        $action = InbankTools::getValue('action', '');

        if ($action == 'add_payment' && $step == 1)
            $this->render_first_step();
        else if (($action == 'add_payment' &&$step == 2) || $action == 'edit_payment')
            $this->render_second_step();

        echo '</div></div>';
    }

    /**
     * Renders first step for ADD payment page.
     */
    public function render_first_step()
    {
        echo '<form method="post" action="options-general.php?page=woocommerce-inbank-settings&action=add_payment">';
        $this->initialize_inbank_select_payment();
        do_settings_sections( 'woocommerce-inbank-settings' );
        echo '<div class="submit">
                <a class="inbank-settings-btn" href="'.admin_url( 'options-general.php?page=woocommerce-inbank-settings').'">'.__('Back to list', 'woocommerce-inbank').'</a>
                <input type="submit" name="submit" id="submit" class="inbank-settings-btn" value="'.__('Next', 'woocommerce-inbank').'">
            </div>';
        echo '</form>';
    }

    /**
     * Renders second step for ADD payment page. This step is also used as EDIT page for payment.
     */
    public function render_second_step()
    {
        $action = InbankTools::getValue('action', '');
        $id_payment = (int)InbankTools::getValue('id_payment', 0);

        if (!in_array($action, ['add_payment', 'edit_payment']))
            $action = '';

        if ($action != 'edit_payment')
            $id_payment = 0;

        $form_url = "options-general.php?page=woocommerce-inbank-settings&action={$action}".($id_payment > 0 ? "&id_payment=".$id_payment : "");

        echo '<form method="post" action="'.$form_url.'">';
        $this->initialize_payment_second_step();
        do_settings_sections( 'woocommerce-inbank-settings' );
        echo '<div class="submit">
                <a class="inbank-settings-btn" href="'.admin_url( 'options-general.php?page=woocommerce-inbank-settings').'">'.__('Back to list', 'woocommerce-inbank').'</a>
                <input type="submit" name="submit" id="submit" class="inbank-settings-btn" value="'.__('Save', 'woocommerce-inbank').'">
            </div>';
        echo '</form>';
    }

    /**
     * First step validation
     * @param $post_data POST data from $_POST
     * @return array|bool
     */
    private function isFirstStepDataValid($post_data)
    {
        $errors = [];

        // check general fields
        if (empty($post_data['class_name']))
            $errors[] = __('No Inbank product selected.', 'woocommerce-inbank');

        if (empty($post_data['country']))
            $errors[] = __('Country not selected.', 'woocommerce-inbank');

        if (empty($post_data['localization_token']))
            $errors[] = __('Localization token is required.', 'woocommerce-inbank');

        if (empty($post_data['localization_id']))
            $errors[] = __('Localization ID is required.', 'woocommerce-inbank');



        // checks if debug fields are filled if debug mode is on
        if (!empty($post_data['debug_mode']))
        {
            if (empty($post_data['api_key_debug']))
                $errors[] = __('Test API key is required.', 'woocommerce-inbank');

            if (empty($post_data['shop_uuid_debug']))
                $errors[] = __('Test shop UUID is required.', 'woocommerce-inbank');

            if (empty($post_data['product_code_debug']))
                $errors[] = __('Test product code is required.', 'woocommerce-inbank');
        }

        // checks if production fields are filled if debug mode is off
        if (empty($post_data['debug_mode']))
        {
            if (empty($post_data['api_key']))
                $errors[] = __('Live API key is required.', 'woocommerce-inbank');

            if (empty($post_data['shop_uuid']))
                $errors[] = __('Live shop UUID is required.', 'woocommerce-inbank');

            if (empty($post_data['product_code']))
                $errors[] = __('Live product code is required.', 'woocommerce-inbank');
        }


        return empty($errors) ? true : $errors;
    }

    /**
     * Initialization of settings of payment ADD second step of EDIT page.
     */
    public function initialize_payment_second_step()
    {
        // available hook list
        $calculator_hooks_list = array(
            'woocommerce_single_product_summary' => __('Product summary', 'woocommerce-inbank'),
            'woocommerce_before_add_to_cart_button' => __('Before add to cart button', 'woocommerce-inbank'),
            'woocommerce_after_add_to_cart_button' => __('After add to cart button', 'woocommerce-inbank'),
            'woocommerce_after_add_to_cart_form' => __('After add to cart form', 'woocommerce-inbank'),
            'woocommerce_product_meta_start' => __('Before product meta data', 'woocommerce-inbank'),
            'woocommerce_product_meta_end' => __('After product meta data', 'woocommerce-inbank'),
            'woocommerce_share' => __('Share section', 'woocommerce-inbank'),
            'woocommerce_inbank_custom' => __('Custom action: woocommerce_inbank_custom', 'woocommerce-inbank'),
        );


        $id_payment = (int)InbankTools::getValue('id_payment', 0);
        $post_data = InbankTools::getValue('woocommerce-inbank_payment', []);
        $db_data = [];

        if ($id_payment > 0) {
            $db_data = new InbankPayment($id_payment);
            if ((int)$db_data->id > 0)
                $db_data = $db_data->toArray();
            else
                $db_data = [];
        }


        $selected_country = self::getFormValue('country', $post_data, $db_data);


        add_settings_section(
            'payment_settings_section',			// ID used to identify this section and with which to register options
            null,		// Title to be displayed on the administration page
            function() { $this->render_custom_form_title('payment_settings_section'); },	// Callback used to render the description of the section
            'woocommerce-inbank-settings'		// Page on which to add this section of options
        );

        InbankPaymentFormHelper::input_hidden_callback(
            array(
                '',
                'inbank_step',
                2,
            )
        );

        add_settings_field(
            'enabled',
            __('Payment enabled', 'woocommerce-inbank'),
            'InbankPaymentFormHelper::input_select_callback',
            'woocommerce-inbank-settings',
            'payment_settings_section',
            array(
                'payment',
                'enabled',
                self::getFormValue('enabled', $post_data, $db_data, 1),
                '',
                array(
                    '1' => __('Yes', 'woocommerce-inbank'),
                    '0' => __('No', 'woocommerce-inbank'))
            )
        );

        add_settings_field(
            'debug_mode',
            __('Enable test mode', 'woocommerce-inbank'),
            'InbankPaymentFormHelper::input_select_callback',
            'woocommerce-inbank-settings',
            'payment_settings_section',
            array(
                'payment',
                'debug_mode',
                self::getFormValue('debug_mode', $post_data, $db_data),
                '',
                array(
                    '1' => __('Yes', 'woocommerce-inbank'),
                    '0' => __('No', 'woocommerce-inbank'))
            )
        );

        InbankPaymentFormHelper::input_hidden_callback(
            array(
                'payment',
                'country',
                $selected_country,
            )
        );

        $class_name = self::getFormValue('class_name', $post_data, $db_data);
        InbankPaymentFormHelper::input_hidden_callback(
            array(
                'payment',
                'class_name',
                $class_name,
            )
        );

        add_settings_field(
            'product_code',
            __('Live product code', 'woocommerce-inbank'),
            'InbankPaymentFormHelper::input_textbox_callback',
            'woocommerce-inbank-settings',
            'payment_settings_section',
            array(
                'payment',
                'product_code',
                self::getFormValue('product_code', $post_data, $db_data),
                '',
            )
        );

        add_settings_field(
            'api_key',
            __('Live API key', 'woocommerce-inbank'),
            'InbankPaymentFormHelper::input_textbox_callback',
            'woocommerce-inbank-settings',
            'payment_settings_section',
            array(
                'payment',
                'api_key',
                self::getFormValue('api_key', $post_data, $db_data),
                __( 'Live API key provided by Inbank.', 'woocommerce-inbank' ),
            )
        );

        add_settings_field(
            'shop_uuid',
            __('Live shop UUID', 'woocommerce-inbank'),
            'InbankPaymentFormHelper::input_textbox_callback',
            'woocommerce-inbank-settings',
            'payment_settings_section',
            array(
                'payment',
                'shop_uuid',
                self::getFormValue('shop_uuid', $post_data, $db_data),
                __( 'Live shop unique identifier provided by Inbank.', 'woocommerce-inbank' ),
            )
        );

        add_settings_field(
            'product_code_debug',
            __('Test product code', 'woocommerce-inbank'),
            'InbankPaymentFormHelper::input_textbox_callback',
            'woocommerce-inbank-settings',
            'payment_settings_section',
            array(
                'payment',
                'product_code_debug',
                self::getFormValue('product_code_debug', $post_data, $db_data),
                '',
            )
        );

        add_settings_field(
            'api_key_debug',
            __('Test API key', 'woocommerce-inbank'),
            'InbankPaymentFormHelper::input_textbox_callback',
            'woocommerce-inbank-settings',
            'payment_settings_section',
            array(
                'payment',
                'api_key_debug',
                self::getFormValue('api_key_debug', $post_data, $db_data),
                __( 'Test API key provided by Inbank.', 'woocommerce-inbank' ),
            )
        );

        add_settings_field(
            'shop_uuid_debug',
            __('Test shop UUID', 'woocommerce-inbank'),
            'InbankPaymentFormHelper::input_textbox_callback',
            'woocommerce-inbank-settings',
            'payment_settings_section',
            array(
                'payment',
                'shop_uuid_debug',
                self::getFormValue('shop_uuid_debug', $post_data, $db_data),
                __( 'Test shop unique identifier provided by Inbank.', 'woocommerce-inbank' ),
            )
        );

        add_settings_field(
            'localization_id',
            __('Localization ID', 'woocommerce-inbank'),
            'InbankPaymentFormHelper::input_textbox_callback',
            'woocommerce-inbank-settings',
            'payment_settings_section',
            array(
                'payment',
                'localization_id',
                self::getFormValue('localization_id', $post_data, $db_data),
                __( 'Localization ID provided by Inbank.', 'woocommerce-inbank' ),
            )
        );

        add_settings_field(
            'localization_token',
            __('Localization token', 'woocommerce-inbank'),
            'InbankPaymentFormHelper::input_textbox_callback',
            'woocommerce-inbank-settings',
            'payment_settings_section',
            array(
                'payment',
                'localization_token',
                self::getFormValue('localization_token', $post_data, $db_data),
                __( 'Localization token provided by Inbank.', 'woocommerce-inbank' ),
            )
        );

        add_settings_field(
            'use_logo',
            __('Display logo', 'woocommerce-inbank'),
            'InbankPaymentFormHelper::input_select_callback',
            'woocommerce-inbank-settings',
            'payment_settings_section',
            array(
                'payment',
                'use_logo',
                self::getFormValue('use_logo', $post_data, $db_data),
                __('Whether the logo appears next to the payment method at checkout.', 'woocommerce-inbank'),
                array(
                    '1' => __('Yes', 'woocommerce-inbank'),
                    '0' => __('No', 'woocommerce-inbank'))
            )
        );

        add_settings_field(
            'logo_size',
            __('Logo size', 'woocommerce-inbank'),
            'InbankPaymentFormHelper::input_select_callback',
            'woocommerce-inbank-settings',
            'payment_settings_section',
            array(
                'payment',
                'logo_size',
                self::getFormValue('logo_size', $post_data, $db_data),
                '',
                array(
                    'small' => __('Small', 'woocommerce-inbank'),
                    'medium' => __('Medium', 'woocommerce-inbank'))
            )
        );

        add_settings_field(
            'custom_logo',
            __('Custom logo', 'woocommerce-inbank'),
            'InbankPaymentFormHelper::input_media_callback',
            'woocommerce-inbank-settings',
            'payment_settings_section',
            array(
                'payment',
                'custom_logo',
                self::getFormValue('custom_logo', $post_data, $db_data),
                '',
            )
        );

        if (in_array($class_name, ['InbankHirePurchaseEE', 'InbankHirePurchaseLV', 'InbankHirePurchasePL', 'InbankHirePurchaseCZ', 'InbankGreenEnergyCZ', 'InbankIndivySliceEE', 'InbankIndivyGoEE', 'InbankGreenEnergyPL', 'InbankSplitIntoPartsLV']))
        {
            add_settings_field(
                'calc_small',
                __('Enable calculations', 'woocommerce-inbank'),
                'InbankPaymentFormHelper::input_select_callback',
                'woocommerce-inbank-settings',
                'payment_settings_section',
                array(
                    'payment',
                    'calc_small',
                    self::getFormValue('calc_small', $post_data, $db_data, 1),
                    __('Whether calculated monthly amounts should be presented on product pages.', 'woocommerce-inbank'),
                    array(
                        '1' => __('Yes', 'woocommerce-inbank'),
                        '0' => __('No', 'woocommerce-inbank')
                    )
                )
            );

            add_settings_field(
                'calc_small_position',
                __('Calculation position', 'woocommerce-inbank'),
                'InbankPaymentFormHelper::input_select_callback',
                'woocommerce-inbank-settings',
                'payment_settings_section',
                array(
                    'payment',
                    'calc_small_position',
                    self::getFormValue('calc_small_position', $post_data, $db_data),
                    '',
                    $calculator_hooks_list
                )
            );
        }

        /*add_settings_field(
            'calc_large',
            __('Enable large calculator', 'woocommerce-inbank'),
            'InbankPaymentFormHelper::input_select_callback',
            'woocommerce-inbank-settings',
            'payment_settings_section',
            array(
                'payment',
                'calc_large',
                self::getFormValue('calc_large', $post_data, $db_data),
                '',
                array(
                    '1' => __('Yes', 'woocommerce-inbank'),
                    '0' => __('No', 'woocommerce-inbank'))
            )
        );*/

        InbankPaymentFormHelper::input_hidden_callback(
            array(
                'payment',
                'calc_large',
                self::getFormValue('calc_large', $post_data, $db_data, 0),
            )
        );

        $original_title = self::getFormValue('title', $post_data, $db_data, []);
        $original_title = array_map(function ($val) { return __('Provided by Inbank:', 'woocommerce-inbank').' '.$val; }, $original_title);

        add_settings_field(
            'custom_title',
            __('Title', 'woocommerce-inbank'),
            'InbankPaymentFormHelper::input_textbox_multilang_callback',
            'woocommerce-inbank-settings',
            'payment_settings_section',
            array(
                'payment',
                'custom_title',
                self::getFormValue('custom_title', $post_data, $db_data),
                $original_title,
                !in_array($class_name, ['InbankHirePurchaseEE', 'InbankHirePurchaseLV', 'InbankHirePurchasePL', 'InbankHirePurchaseCZ', 'InbankGreenEnergyCZ', 'InbankGreenEnergyPL']) ? 'readonly' : '',
                strtoupper($selected_country)
            )
        );

        add_settings_field(
            'minimum_order_amount',
            __('Minimum order amount', 'woocommerce-inbank'),
            'InbankPaymentFormHelper::input_textbox_callback',
            'woocommerce-inbank-settings',
            'payment_settings_section',
            array(
                'payment',
                'minimum_order_amount',
                self::getFormValue('minimum_order_amount', $post_data, $db_data),
                '',
                'readonly'
            )
        );

        add_settings_field(
            'maximum_order_amount',
            __('Maximum order amount', 'woocommerce-inbank'),
            'InbankPaymentFormHelper::input_textbox_callback',
            'woocommerce-inbank-settings',
            'payment_settings_section',
            array(
                'payment',
                'maximum_order_amount',
                self::getFormValue('maximum_order_amount', $post_data, $db_data),
                '',
                'readonly'
            )
        );

        add_settings_field(
            'period',
            __('Period', 'woocommerce-inbank'),
            'InbankPaymentFormHelper::input_textbox_callback',
            'woocommerce-inbank-settings',
            'payment_settings_section',
            array(
                'payment',
                'period',
                self::getFormValue('period', $post_data, $db_data),
                '',
                'readonly'
            )
        );

        InbankPaymentFormHelper::input_hidden_callback(
            array(
                'payment',
                'default_period',
                self::getFormValue('default_period', $post_data, $db_data),
            )
        );

        InbankPaymentFormHelper::input_hidden_callback(
            array(
                'payment',
                'needs_approval',
                self::getFormValue('needs_approval', $post_data, $db_data),
            )
        );

        InbankPaymentFormHelper::input_hidden_callback(
            array(
                'payment',
                'default_payment_day',
                self::getFormValue('default_payment_day', $post_data, $db_data),
            )
        );

        InbankPaymentFormHelper::input_hidden_callback(
            array(
                'payment',
                'down_payment',
                self::getFormValue('down_payment', $post_data, $db_data),
            )
        );

        InbankPaymentFormHelper::input_hidden_callback(
            array(
                'payment',
                'max_period',
                self::getFormValue('max_period', $post_data, $db_data),
            )
        );

        InbankPaymentFormHelper::input_multilang_hidden_callback(
            array(
                'payment',
                'granted',
                self::getFormValue('granted', $post_data, $db_data),
            )
        );

        InbankPaymentFormHelper::input_multilang_hidden_callback(
            array(
                'payment',
                'title',
                self::getFormValue('title', $post_data, $db_data),
            )
        );

        InbankPaymentFormHelper::input_multilang_hidden_callback(
            array(
                'payment',
                'additional_details',
                self::getFormValue('additional_details', $post_data, $db_data),
            )
        );

        add_settings_field(
            'custom_additional_details',
            __('Additional details', 'woocommerce-inbank'),
            'InbankPaymentFormHelper::input_textarea_multilang_callback',
            'woocommerce-inbank-settings',
            'payment_settings_section',
            array(
                'payment',
                'custom_additional_details',
                self::getFormValue('custom_additional_details', $post_data, $db_data),
                '',
                '',
                strtoupper($selected_country)
            )
        );

        add_settings_field(
            'restrictions_categories',
            __('Apply payment option only for these categories', 'woocommerce-inbank'),
            'InbankPaymentFormHelper::post_categories_meta_box',
            'woocommerce-inbank-settings',
            'payment_settings_section',
            array(
                'payment',
                'restrictions_selected_categories',
                [
                    'args' => ['taxonomy' => 'product_cat'],
                    'value' => self::getFormValue('restrictions_selected_categories', $post_data, $db_data)
                ]
            )
        );

        InbankPaymentFormHelper::input_hidden_callback(
            array(
                'payment',
                'restrictions_selected_products[]',
            )
        );

        add_settings_field(
            'restrictions_products',
            __('Apply payment option only for these products', 'woocommerce-inbank'),
            'InbankPaymentFormHelper::product_select_autocomplete',
            'woocommerce-inbank-settings',
            'payment_settings_section',
            array(
                'payment',
                'restrictions_selected_products',
                self::getFormValue('restrictions_selected_products', $post_data, $db_data)
            )
        );

        add_settings_field(
            'restrictions_condition',
            __('Restrictions apply method', 'woocommerce-inbank'),
            'InbankPaymentFormHelper::input_select_callback',
            'woocommerce-inbank-settings',
            'payment_settings_section',
            array(
                'payment',
                'restrictions_condition',
                self::getFormValue('restrictions_condition', $post_data, $db_data),
                '',
                ['AND' => 'AND (From selected category AND product list)', 'OR' => 'OR (From selected category OR product list)'],
                'style="width:330px;"'
            )
        );


    }

    public function initialize_inbank_select_payment()
    {
        $selected_country = InbankTools::getValue('country', '');
        $post_data = InbankTools::getValue('woocommerce-inbank_payment', []);

        if (empty($selected_country))
            $selected_country = isset($post_data['country']) ? $post_data['country'] : '';

        $countries_available = $this->getCountriesByProducts();

        $payments_available = InbankPaymentFormHelper::filterPayments($selected_country);

        // prefillling data
        if (!empty($selected_country))
        {
            $prefill_data = InbankPaymentFormHelper::getPaymentDataToPrefill($selected_country);
            $prefill_data = array_filter($prefill_data);
            if (!empty($prefill_data))
                $post_data = array_merge($post_data, $prefill_data);
        }

        add_settings_section(
            'inbank_select_payment',			// ID used to identify this section and with which to register options
            null,		// Title to be displayed on the administration page
            function() { $this->render_custom_form_title('inbank_select_payment'); },	// Callback used to render the description of the section
            'woocommerce-inbank-settings'		// Page on which to add this section of options
        );

        InbankPaymentFormHelper::input_hidden_callback(
            array(
                '',
                'inbank_step',
                '1',
            )
        );

        add_settings_field(
            'country',
            __('Country', 'woocommerce-inbank'),
            'InbankPaymentFormHelper::input_select_callback',
            'woocommerce-inbank-settings',
            'inbank_select_payment',
            array(
                'payment',
                'country',
                $selected_country,
                __('Select country where you provide your services.', 'woocommerce-inbank'),
                $countries_available
            )
        );

        add_settings_field(
            'class_name',
            __('Product', 'woocommerce-inbank'),
            'InbankPaymentFormHelper::input_select_callback',
            'woocommerce-inbank-settings',
            'inbank_select_payment',
            array(
                'payment',
                'class_name',
                self::getFormValue('class_name', $post_data),
                __('Select Inbank product.', 'woocommerce-inbank'),
                $payments_available,
                empty($payments_available) ? 'disabled' : ''
            )
        );

        add_settings_field(
            'debug_mode',
            __('Enable test mode', 'woocommerce-inbank'),
            'InbankPaymentFormHelper::input_select_callback',
            'woocommerce-inbank-settings',
            'inbank_select_payment',
            array(
                'payment',
                'debug_mode',
                self::getFormValue('debug_mode', $post_data, ['debug_mode' => '0']),
                '',
                array(
                    '1' => __('Yes', 'woocommerce-inbank'),
                    '0' => __('No', 'woocommerce-inbank')
                ),
                empty($payments_available) ? 'disabled' : ''
            )
        );

        add_settings_field(
            'product_code',
            __('Live product code', 'woocommerce-inbank'),
            'InbankPaymentFormHelper::input_textbox_callback',
            'woocommerce-inbank-settings',
            'inbank_select_payment',
            array(
                'payment',
                'product_code',
                '',
                __( 'Live product code provided by Inbank.', 'woocommerce-inbank' ),
                empty($payments_available) ? 'disabled' : ''
            )
        );


        add_settings_field(
            'api_key',
            __('Live API key', 'woocommerce-inbank'),
            'InbankPaymentFormHelper::input_textbox_callback',
            'woocommerce-inbank-settings',
            'inbank_select_payment',
            array(
                'payment',
                'api_key',
                self::getFormValue('api_key', $post_data),
                __( 'Live API key provided by Inbank.', 'woocommerce-inbank' ),
            )
        );

        add_settings_field(
            'shop_uuid',
            __('Live shop UUID', 'woocommerce-inbank'),
            'InbankPaymentFormHelper::input_textbox_callback',
            'woocommerce-inbank-settings',
            'inbank_select_payment',
            array(
                'payment',
                'shop_uuid',
                self::getFormValue('shop_uuid', $post_data),
                __( 'Live shop Unique Identifier provided by Inbank.', 'woocommerce-inbank' ),
            )
        );

        add_settings_field(
            'product_code_debug',
            __('Test product code', 'woocommerce-inbank'),
            'InbankPaymentFormHelper::input_textbox_callback',
            'woocommerce-inbank-settings',
            'inbank_select_payment',
            array(
                'payment',
                'product_code_debug',
                '',
                __( 'Test product code provided by Inbank.', 'woocommerce-inbank' ),
                empty($payments_available) ? 'disabled' : ''
            )
        );


        add_settings_field(
            'api_key_debug',
            __('Test API key', 'woocommerce-inbank'),
            'InbankPaymentFormHelper::input_textbox_callback',
            'woocommerce-inbank-settings',
            'inbank_select_payment',
            array(
                'payment',
                'api_key_debug',
                self::getFormValue('api_key_debug', $post_data),
                __( 'Test API key provided by Inbank.', 'woocommerce-inbank' ),
            )
        );

        add_settings_field(
            'shop_uuid_debug',
            __('Test shop UUID', 'woocommerce-inbank'),
            'InbankPaymentFormHelper::input_textbox_callback',
            'woocommerce-inbank-settings',
            'inbank_select_payment',
            array(
                'payment',
                'shop_uuid_debug',
                self::getFormValue('shop_uuid_debug', $post_data),
                __( 'Test shop Unique Identifier provided by Inbank.', 'woocommerce-inbank' ),
            )
        );

        add_settings_field(
            'localization_id',
            __('Localization ID', 'woocommerce-inbank'),
            'InbankPaymentFormHelper::input_textbox_callback',
            'woocommerce-inbank-settings',
            'inbank_select_payment',
            array(
                'payment',
                'localization_id',
                self::getFormValue('localization_id', $post_data),
                __( 'Localization ID provided by Inbank.', 'woocommerce-inbank' ),
            )
        );

        add_settings_field(
            'localization_token',
            __('Localization token', 'woocommerce-inbank'),
            'InbankPaymentFormHelper::input_textbox_callback',
            'woocommerce-inbank-settings',
            'inbank_select_payment',
            array(
                'payment',
                'localization_token',
                self::getFormValue('localization_token', $post_data),
                __( 'Localization token provided by Inbank.', 'woocommerce-inbank' ),
            )
        );

    }

    /**
     * Function returns countries available by Inbank products.
     * @return array
     */
    private function getCountriesByProducts()
    {
        $all_countries = WC()->countries->get_countries();

        $countries_by_payments = InbankPaymentFormHelper::getAllProductCountries();
        $final_countries = [];
        $final_countries[''] = __('Select country', 'woocommerce-inbank');
        foreach ($countries_by_payments as $country)
        {
            $country = strtoupper($country);
            if (isset($all_countries[$country]))
                $final_countries[$country] = $all_countries[$country];
        }

        return $final_countries;
    }

    /**
     * Get form value from possible data sources.
     * @param $field_name
     * @param null $post_data Data from $_POST
     * @param null $db_data Data from database table.
     * @return mixed|string
     */
    private static function getFormValue($field_name, $post_data = null, $db_data = null, $default_value = '')
    {
        return isset($post_data[$field_name]) ? $post_data[$field_name] : (isset($db_data[$field_name]) ? $db_data[$field_name] : $default_value);
    }

    public function initialize_logs()
    {
        $list = InbankLogs::getLogs(null, 200);

        $html = '<div class="tableFixHead">
                  <table>
                    <thead>
                      <tr>
                        <th width="10%">Date</th>
                        <th width="5%">Order ID</th>
                        <th>Result</th>
                        <th>ResultInfo</th>
                        <th>ResultMsg</th>
                      </tr>
                    </thead>
                    <tbody>';

        foreach ($list as $row)
        {
            $html .= "<tr>
                        <td>{$row['date_add']}</td>
                        <td>{$row['id_order']}</td>
                        <td>{$row['result']}</td>
                        <td>{$row['resultInfo']}</td>
                        <td>{$row['resultMsg']}</td>
                      </tr>";
        }

        $html .=   '</tbody>
                  </table>
                </div>';

        echo $html;
    }

    public function validate_api_settings ($input)
    {
        if (isset($input['api_user']) && isset($input['api_pass']))
        {
            $api_class = 'InbankAPI';
            if (class_exists($api_class) && method_exists($api_class, 'testIfAPIUsernameAndPasswordIsValid'))
            {
                $is_valid = call_user_func($api_class.'::testIfAPIUsernameAndPasswordIsValid', $input['api_user'], $input['api_pass']);

                if (!$is_valid)
                {
                    add_settings_error('my_option_notice', 'my_option_notice', __('API username or password is not valid.'), 'error');
                }
            }
        }

        return $input;
    }

    /**
     * Payment enable/disable function.
     */
    public function enable_disable_payment()
    {
        $id_payment = (int)InbankTools::getValue('id_payment');
        $sql = "UPDATE `".InbankFrameworkBridge::getDbPrefix()."inbank_payment` SET `enabled` = (CASE WHEN enabled = 1 THEN 0 ELSE 1 END) WHERE `id_payment` = ".$id_payment;
        if ($id_payment > 0)
            InbankFrameworkBridge::queryDb($sql);

        wp_redirect( admin_url( 'options-general.php?page=woocommerce-inbank-settings&conf=14' ) );
    }

    /**
     * Payment delete fuction.
     */
    public function delete_payment()
    {
        $id_payment = (int)InbankTools::getValue('id_payment');

        if ($id_payment > 0)
        {
            $paymentObj = new InbankPayment($id_payment);

            if ($paymentObj->id > 0) {
                if ($paymentObj->delete()) {
                    $files = glob(INBANK_MODULE_DIR . 'cache/' . $id_payment . '_*.html');

                    foreach ($files as $file) {
                        @unlink($file);
                    }

                    wp_redirect(admin_url('options-general.php?page=woocommerce-inbank-settings&conf=12'));
                    return;
                }
            }
        }

        wp_redirect( admin_url( 'options-general.php?page=woocommerce-inbank-settings&warn=13' ) );
    }

    /**
     * Force update for all payment products.
     */
    public function update_products()
    {
        $scanned_directory = array_diff(scandir(InbankFrameworkBridge::getCacheDir()), array('..', '.', 'index.php'));
        foreach ($scanned_directory as $file) {
            unlink(InbankFrameworkBridge::getCacheDir().$file);
        }
        InbankPayment::updateAllProducts();
        wp_redirect( admin_url( 'options-general.php?page=woocommerce-inbank-settings&conf=17' ) );
    }

    /**
     * Force update for all payment products.
     */
    public function force_update_db()
    {
        // resetting version
        update_option('woocommerce-inbank_last_update_check_version', '1.0.0');
        // updating plugin
        Woocommerce_Inbank_Updater::update();

        // redirect back to settings page
        wp_redirect( admin_url( 'options-general.php?page=woocommerce-inbank-settings&conf=19' ) );
    }

    /**
     * Renders custom title for settings forms.
     * @param $section Section name
     */
    public function render_custom_form_title($section)
    {
        switch($section) {
            case 'payment_settings_section':
            case 'inbank_select_payment':
                echo "<br/>";
                break;
            case 'global_settings_section':
                echo "<p class=\"inbank-block-title\">" . __('General settings', 'woocommerce-inbank') . "</p>";
                break;
        }
    }
}