<?php

if ( ! defined( 'ABSPATH' ) ) {
    exit;
}

class WC_Inbank_Payment_Gateway extends WC_Payment_Gateway
{
    private $CALLBACK_URL = "inbank_callback";
    private $CANCEL_URL = "inbank_cancel";
    private $RETURN_URL = "inbank_return";
    private $min_amount = 0;

    private $currency;
    private $country;
    private $order_sum;

    /**
     * @var $payment InbankPayment
     */
    private $payment;

    /**
     * @var $product InbankBalticsProduct
     */
    private $product;

    /**
     * Class constructor
     */
    public function __construct($id_payment = 0)
    {
        $this->id = 'inbank';
        $this->has_fields = true;
        $this->method_title = 'Inbank';
        $this->method_description = __('Enable payment on your website with Inbank', 'woocommerce-inbank');

        $this->title = $this->method_title;
        $this->description = $this->method_description;
        //$this->icon = '';
        $this->supports = array(
            'products'
        );

        if ($id_payment == 0)
            $id_payment = (int)str_replace('inbank_', '', InbankTools::getValue('id_payment', ''));


        $this->init_payment((int)$id_payment);

        // Payment listener/API hook
        add_action( 'woocommerce_api_'.$this->CALLBACK_URL.'_'.$id_payment, array( $this, 'inbank_callback' ) );
        add_action( 'woocommerce_api_'.$this->CANCEL_URL.'_'.$id_payment, array( $this, 'inbank_cancel' ) );
        add_action( 'woocommerce_api_'.$this->RETURN_URL.'_'.$id_payment, array( $this, 'inbank_return' ) );
    }


    public function get_icon()
    {
        if (empty($this->payment->use_logo))
            return '';

        $icon_size = $this->payment->logo_size;
        $style = '';

        if (!empty($this->payment->custom_logo))
            $icon_url = wp_get_attachment_image_url($this->payment->custom_logo);

        if (!isset($icon_url) || empty($icon_url))
            $icon_url = WC_HTTPS::force_https_url( $this->icon );
        else
            $style = 'max-height: '.($icon_size == 'small' ? 26 : 36).'px;';

        $icon  = '<img src="' . $icon_url . '" alt="'.$this->get_title().'" style="'.$style.'"/>';

        return apply_filters( 'woocommerce_gateway_icon', $icon, $this->id );
    }

    public function payment_fields()
    {
        echo wptexturize( $this->get_description() );
    }

    public function init_payment($id_payment)
    {
        if($id_payment > 0)
        {
            $this->payment = new InbankPayment($id_payment);
            $icon_path = null;

            if ($this->payment->id > 0)
            {
                $class_name = $this->payment->class_name;
                if (!class_exists($class_name))
                    return;

                $lang = strtoupper(InbankFrameworkBridge::getLocaleIso());
                $payment_countries = $class_name::getAvailableCountries();

                if ($this->payment->use_logo) {
                    $this->product = new $class_name($this->payment);
                    $icon = $this->product->getLogoPath(true);
                    if (is_array($icon))
                        $icon_path = $icon['url'];
                    else
                        $icon_path = $icon;
                }

                $this->id = 'inbank_'.$id_payment;
                $this->icon = $icon_path;
                $this->method_title = $class_name::getDefaultTitle() . ' (' . InbankFrameworkBridge::getCountryByIso(reset($payment_countries)) .')';
                $this->title = isset($this->payment->title[$lang]) ? $this->payment->title[$lang] : $this->method_title;
                $this->enabled = (int)$this->payment->enabled === 1 ? 'yes' : 'no';
                $this->inbank_payment = $this->payment;

                if (!is_admin())
                {
                    if($this->enabled == 'yes' && !$this->is_available_at_checkout())
                    {
                        $this->enabled = 'no';
                    }

                    if($this->enabled == 'yes' && !InbankPayment::is_payment_available_by_restrictions($id_payment))
                    {
                        $this->enabled = 'no';
                    }

                    if ($this->enabled == 'yes')
                    {
                        if (!isset($this->product))
                            $this->product = new $class_name($this->payment);

                        $currency = InbankFrameworkBridge::getCurrency();
                        $order_total = $this->getOrderSum();

                        $payment_method_rendered = $this->product->renderPaymentForCheckout($order_total, $currency, $lang);

                        if (!empty($payment_method_rendered) && $order_total > 0) {
                            if (is_array($payment_method_rendered['title'])) {
                                if (isset($payment_method_rendered['title'][$lang])) {
                                    $this->title = $payment_method_rendered['title'][$lang];
                                } elseif (!isset($payment_method_rendered['title'][$lang])) {
                                    $this->title = reset($payment_method_rendered['title']);
                                }
                            } else {
                                $this->title = $payment_method_rendered['title'];
                            }

                            $this->description = $payment_method_rendered['description'];
                        }
                        else
                        {
                            $this->enabled = 'no';
                            $this->title = $class_name::getDefaultTitle();
                            $this->description = '';
                        }
                    }
                    else
                    {
                        $this->title = $class_name::getDefaultTitle();
                        $this->description = '';
                    }
                }

                $this->settings = [
                    'title' => $this->title,
                    'description' => $this->description,
                    'enabled' => $this->enabled,
                ];

                if (!is_admin())
                {
                    //add_action('woocommerce_thankyou_'.$this->id, array($this, 'thankyou_page'));
                }
            }
        }
    }

    public function get_option($key, $empty_value = null)
    {
        if ( empty( $this->settings ) ) {
            $this->init_settings();
        }

        // Get option default if unset.
        if ( ! isset( $this->settings[ $key ] ) ) {
            $this->settings[ $key ] = '';
        }

        if ( ! is_null( $empty_value ) && '' === $this->settings[ $key ] ) {
            $this->settings[ $key ] = $empty_value;
        }

        return $this->settings[ $key ];
    }

    public function update_option($key, $value = '')
    {
        if ($key === 'enabled')
        {
            $enabled = $value === 'yes' ? 1 : 0;
            $this->settings[ $key ] = $value;
            $this->enabled = $value;
            InbankFrameworkBridge::updateDb('inbank_payment', ['enabled' => $enabled], ['id_payment' => (int)str_replace('inbank_', '', $this->id)]);
        }

        return true;
    }

    public function init_settings()
    {
        $this->settings = [
            'title' => $this->title,
            'description' => $this->description,
            'enabled' => $this->enabled,
        ];
    }


    public function process_payment($order_id)
    {
        // key for return and alert callback urls
        $hash = md5('woocommerce-inbank'.AUTH_SALT);
        $order = wc_get_order( $order_id );

        if (
            method_exists($order, 'get_billing_first_name') &&
            method_exists($order, 'get_billing_last_name') &&
            method_exists($order, 'get_billing_phone') &&
            method_exists($order, 'get_billing_email') &&
            method_exists($order, 'get_billing_address_1') &&
            method_exists($order, 'get_billing_address_2') &&
            method_exists($order, 'get_billing_city')
        )
        {
            $order_first_name = $order->get_billing_first_name();
            $order_last_name = $order->get_billing_last_name();
            $order_client_phone = $order->get_billing_phone();
            $order_client_email = $order->get_billing_email();
            $order_address = $order->get_billing_address_1().' '.$order->get_billing_address_2().', '.$order->get_billing_city();
        }
        else
        {
            $address = $order->get_address();
            $order_first_name = $address['first_name'];
            $order_last_name = $address['last_name'];
            $order_client_phone = $address['phone'];
            $order_client_email = $address['email'];
            $order_address = $address['address_1'].' '.$address['address_2'].', '.$address['city'];
        }

        $endpoint = "pos_sessions";

        $request_body = [];
        //general info
        $request_body['product_code'] = $this->product->getProductCode();
        $request_body['total_amount'] = (float)wc_format_decimal( $order->get_total(), 2 );;
        $request_body['currency'] = $order->get_currency();
        $request_body['locale'] = InbankFrameworkBridge::getLocale();

        //customer data
        if (!empty($order_first_name) && !empty($order_last_name)) {
            $request_body['customer_data']['first_name'] = $order_first_name;
            $request_body['customer_data']['last_name'] = $order_last_name;
        }

        if (!empty($order_client_email) && !empty($order_client_phone)) {
            $request_body['customer_contact_data']['email'] = $order_client_email;
            $request_body['customer_contact_data']['mobile'] = $order_client_phone;
        }

        //order info
        $request_body['purchase']['purchase_reference'] = $order_id;
        //merchant info
        $request_body['purchase']['merchant']['merchant_name'] = get_option('blogname');
        $request_body['purchase']['merchant']['merchant_domain_name'] = get_site_url();
        $request_body['purchase']['merchant']['merchant_order_reference'] = $order_id;
        //return urls
        //$request_body['partner_urls']['return_url'] = WC()->api_request_url($this->RETURN_URL.'_'.$this->payment->id);
        //$request_body['partner_urls']['cancel_url'] = WC()->api_request_url($this->CANCEL_URL.'_'.$this->payment->id);
        //$request_body['partner_urls']['callback_url'] = WC()->api_request_url($this->CALLBACK_URL.'_'.$this->payment->id).'?token='.$hash;

        $request_body['partner_urls']['return_url'] = self::api_request_url($this->RETURN_URL.'_'.$this->payment->id);
        $request_body['partner_urls']['cancel_url'] = self::api_request_url($this->CANCEL_URL.'_'.$this->payment->id);
        $request_body['partner_urls']['callback_url'] = self::api_request_url($this->CALLBACK_URL.'_'.$this->payment->id, null, $hash);

        $request_body['integration_info']['method'] = 'plugin';
        $request_body['integration_info']['platform'] = 'wordpress';
        $request_body['integration_info']['version'] = WOOCOMMERCE_INBANK_VERSION;

        $response = $this->product->api->initiateSession($request_body);
        if (isset($response->uuid)) {
            // adding order info to Inbank table
            $details = InbankDetail::getDetailsByOrderId($order_id);
            if (empty($details))
            {
                $details = new InbankDetail();
            }

            $details->id_order = $order_id;
            $details->id_payment = $this->payment->id;
            $details->session_uuid = $response->uuid;
            $details->save();

            InbankLogs::addLog(
                $order_id,
                'Customer redirected to Inbank ePOS system.',
                InbankLogs::TYPE_INFO,
                $this->product->getProductCode(),
                $endpoint,
                json_encode($response)
            );

            // Reduce stock levels
            if ( version_compare( WC_VERSION, '3.0', '<' ) ) {
                if ( ! get_post_meta( $order_id, '_order_stock_reduced', true ) ) {
                    $order->reduce_order_stock();
                }
            } elseif (function_exists('wc_maybe_reduce_stock_levels')) {
                wc_maybe_reduce_stock_levels( $order->get_id() );
            }

            // order state change
            $new_order_status_wc = self::inbank_state_to_wc_state('pending');
            $order->update_status($new_order_status_wc['status'], $new_order_status_wc['message']);

            // Remove cart
            WC()->cart->empty_cart();

            return array(
                'result'   => 'success',
                'redirect' => $response->redirect_url
            );

        } else {
            InbankLogs::addLog(
                $order_id,
                'Bad response from server.',
                InbankLogs::TYPE_ERROR,
                $this->product->getProductCode(),
                $endpoint,
                json_encode($response)
            );

            return array(
                'result'   => 'success',
                'redirect' => wc_get_cart_url()
            );
        }
    }

    public function inbank_callback()
    {
        // Inbank response
        $token = InbankTools::getValue('token');
        $hmac = InbankTools::getValue('hmac');
        $timestamp = InbankTools::getValue('timestamp');
        $message = json_decode(InbankTools::getValue('message'));
        $id_order = isset($message->purchase_reference) ? (int)$message->purchase_reference : 0;
        $hash = md5('woocommerce-inbank'.AUTH_SALT);

        if ($token !== $hash) {
            http_response_code(422);
            die('Bad token');
        }

        // DEBUG
        if (WOOCOMMERCE_INBANK_DEBUG)
        {
            $debug_data = ['session_uuid' => $message->uuid, 'order_id' => $id_order, 'hmac' => $hmac, 'message' => $message, '$_POST' => $_POST, '$_GET' => $_GET, '$_SERVER' => $_SERVER];
            file_put_contents(plugin_dir_path(dirname(__FILE__)).'debug/response_callback_'.date('Ymd_His').'.txt', json_encode($debug_data));
        }

        if ($id_order == 0) {
            InbankLogs::addLog(
                0,
                __('There was a problem trying to find the correct order from Inbank callback.', 'woocommerce-inbank'),
                InbankLogs::TYPE_ERROR,
                '',
                'Callback',
                json_encode($message)
            );
            http_response_code(422);
            die("There was a problem trying to find the correct order from Inbank callback");
        }

        $details = InbankDetail::getDetailsByOrderId($id_order);
        if (empty($details))
        {
            InbankLogs::addLog(
                $id_order,
                __('Cannot find Inbank Order detail by order id.', 'woocommerce-inbank'),
                InbankLogs::TYPE_ERROR,
                '',
                'Callback'
            );
            http_response_code(422);
            die("Cannot find Inbank Order detail by order id.");
        }

        $id_payment = $details->id_payment;

        $payment = new InbankPayment($id_payment);
        $current_key = $payment->debug_mode ? $payment->api_key_debug : $payment->api_key;
        if ($message->uuid !== $details->session_uuid ||
            !InbankTools::verifyHmac(
                $hmac,
                $timestamp,
                stripslashes(InbankTools::getValue('message')),
                $current_key
            )
        ) {
            InbankLogs::addLog(
                $id_order,
                __('Failed to verify hmac or local session ID doesn\'t match the ID we received in callback.', 'woocommerce-inbank'),
                InbankLogs::TYPE_ERROR,
                '',
                'CallbackFrontController',
                json_encode($message)
            );
            http_response_code(422);
            die("Failed to verify hmac or local session id doesn't match the id we received");
        }
        $new_order_status = $message->status;

        //loading up inbank product
        $class_name = $payment->class_name;
        $inbank_product = new $class_name($payment);

        //Checking payment info. some inbank products don't need this checking, therefore they won't have this method.
        if ($payment->needs_approval && method_exists($class_name, "checkPaymentStatus")) {
            try {
                $new_status = $inbank_product->checkPaymentStatus($details);
                if (isset($new_status->credit_contract_uuid)) {
                    $details->contract_uuid = $new_status->credit_contract_uuid;
                    $details->save();
                }
                $new_order_status = $new_status->status;
            } catch (Exception $e) {
                //don't need to end processing here, we can continue and try to get cotract uuid later.
                InbankLogs::addLog(
                    $id_order,
                    __('Something went wrong trying to get contract ID. Check the order manually.', 'woocommerce-inbank'),
                    InbankLogs::TYPE_WARNING,
                    '',
                    'CallbackFrontController',
                    $e->getMessage()
                );
            }
        }

        // second request is made to get newest data from DB as few concurrent requests may be performing the same action
        $order = wc_get_order( $id_order );
        $current_order_state_wc = 'wc-' . $order->get_status();
        $new_order_status_wc = self::inbank_state_to_wc_state($new_order_status);
        $cancelled_order_state = self::inbank_state_to_wc_state('cancelled');
        $options = get_option( 'woocommerce-inbank_global_settings' );
        //checking if status changed
        if ($new_order_status !== 'pending' && $current_order_state_wc != $cancelled_order_state['status'] && $new_order_status_wc['status'] !== $current_order_state_wc) {
            try {
                if (
                    $new_order_status_wc['status'] == $options['order_state_confirmed'] &&
                    in_array($new_order_status_wc['status'], [$options['order_state_pending'], $options['order_state_granted']]))
                {
                    $order->payment_complete();
                    // some notes to customer (replace true with false to make it private)
                    $order->add_order_note($new_order_status_wc['message']);
                }
                else
                    $order->update_status($new_order_status_wc['status'], $new_order_status_wc['message']);
            } catch (Exception $e) {
                InbankLogs::addLog(
                    $id_order,
                    'Failed to change order status.',
                    InbankLogs::TYPE_WARNING,
                    '',
                    'CallbackFrontController',
                    $e->getMessage()
                );
                http_response_code(422);
                die("Failed to change order status.");
            }
        }
        $details->last_state = $new_order_status;
        $details->timestamp = $timestamp;
        $details->save();

        // adding log record
        InbankLogs::addLog(
            $id_order,
            'Got callback for order ' . $id_order . ' with state ' . $new_order_status,
            InbankLogs::TYPE_INFO,
            '',
            'Callback'
        );

        die();
    }

    public function inbank_return()
    {
        $hmac = InbankTools::getValue('hmac');
        $timestamp = InbankTools::getValue('timestamp');
        $message = json_decode(InbankTools::getValue('message'));
        $id_order = isset($message->purchase_reference) ? (int)$message->purchase_reference : 0;

        if ($id_order == 0) {
            InbankLogs::addLog(
                0,
                __('There was a problem trying to find the correct order from Inbank callback.', 'woocommerce-inbank'),
                InbankLogs::TYPE_ERROR,
                '',
                'Cancel'
            );
            wp_redirect( home_url() );
            die();
        }

        //trying to get order id from inbank redirect
        $details = InbankDetail::getDetailsByOrderId($id_order);
        if (empty($details))
        {
            InbankLogs::addLog(
                $id_order,
                __('There was a problem trying to find Inbank Order detail in database.', 'woocommerce-inbank'),
                InbankLogs::TYPE_ERROR,
                '',
                'Cancel'
            );
            wp_redirect( home_url() );
            die();
        }

        $id_payment = $details->id_payment;

        $payment = new InbankPayment($id_payment);
        $current_key = $payment->debug_mode ? $payment->api_key_debug : $payment->api_key;
        if ($message->uuid !== $details->session_uuid
            || !InbankTools::verifyHmac(
                $hmac,
                $timestamp,
                stripslashes(InbankTools::getValue('message')),
                $current_key
            )
        ) {
            InbankLogs::addLog(
                $id_order,
                __('Failed to verify hmac or local session ID doesn\'t match the ID we received in callback.', 'woocommerce-inbank'),
                InbankLogs::TYPE_ERROR,
                '',
                'Return'
            );
            wp_redirect( home_url() );
            die();
        }

        $new_order_status = $message->status;

        //loading up inbank product
        $class_name = $payment->class_name;
        $inbank_product = new $class_name($payment);

        //Checking payment info. some inbank products don't need this checking, therefore they won't have this method.
        //Contract UUID is mandatory for accepting/declining orders with payment_granted state
        if ($payment->needs_approval && method_exists($class_name, "checkPaymentStatus")) {
            try {
                $new_status = $inbank_product->checkPaymentStatus($details);
                if (isset($new_status->credit_contract_uuid)) {
                    $details->contract_uuid = $new_status->credit_contract_uuid;
                    $details->save();
                }
                $new_order_status = $new_status->status;
            } catch (Exception $e) {
                //don't need to end processing here, we can continue and try to get contract uuid later.
                InbankLogs::addLog(
                    $id_order,
                    __('Inbank product may need merchant approval.', 'woocommerce-inbank'),
                    InbankLogs::TYPE_ERROR,
                    '',
                    'Return',
                    $e->getMessage()
                );
            }
        }

        // adding log record
        InbankLogs::addLog(
            $id_order,
            sprintf(__('Returned from ePOS to order %d with state %s', 'woocommerce-inbank'), $id_order, $new_order_status),
            InbankLogs::TYPE_INFO,
            '',
            'Return'
        );

        $options = get_option( 'woocommerce-inbank_global_settings' );
        $new_order_status_wc = self::inbank_state_to_wc_state($new_order_status, $options);
        $order = wc_get_order( $id_order );
        $current_order_state_wc = 'wc-' . $order->get_status();

        //checking if status changed
        if ($new_order_status !== 'pending' && $current_order_state_wc && $current_order_state_wc !== $new_order_status_wc['status']) {
            try {
                if (
                    $new_order_status_wc['status'] == $options['order_state_confirmed'] &&
                    in_array($new_order_status_wc['status'], [$options['order_state_pending'], $options['order_state_granted']]))
                {
                    $order->payment_complete();
                    // some notes to customer (replace true with false to make it private)
                    $order->add_order_note($new_order_status_wc['message']);
                }
                else
                    $order->update_status($new_order_status_wc['status'], $new_order_status_wc['message']);
            } catch (Exception $e) {
                InbankLogs::addLog(
                    $id_order,
                    'Failed to change order status.',
                    InbankLogs::TYPE_ERROR,
                    '',
                    'Return',
                    $e->getMessage()
                );
            }

            try {
                $details->last_state = $new_order_status;
                $details->timestamp = $timestamp;
                $details->save();
            } catch (Exception $e) {
                InbankLogs::addLog(
                    $id_order,
                    'Failed to save Inbank details.',
                    InbankLogs::TYPE_ERROR,
                    '',
                    'Return',
                    $e->getMessage()
                );
            }
        }

        if ($new_order_status === 'cancelled' || $new_order_status === 'declined') {
            wp_redirect( home_url() );
            die();
        }

        wp_redirect( $this->get_return_url($order) );
        die();
    }

    public function inbank_cancel()
    {
        $hmac = InbankTools::getValue('hmac');
        $timestamp = InbankTools::getValue('timestamp');
        $message = json_decode(InbankTools::getValue('message'));
        $id_order = isset($message->purchase_reference) ? (int)$message->purchase_reference : 0;

        if ($id_order == 0) {
            InbankLogs::addLog(
                0,
                __('There was a problem trying to find the correct order from Inbank callback.', 'woocommerce-inbank'),
                InbankLogs::TYPE_ERROR,
                '',
                'Cancel'
            );
            wp_redirect( home_url() );
            die();
        }

        //trying to get order id from inbank redirect
        $details = InbankDetail::getDetailsByOrderId($id_order);
        if (empty($details))
        {
            InbankLogs::addLog(
                $id_order,
                __('There was a problem trying to find Inbank Order detail in database.', 'woocommerce-inbank'),
                InbankLogs::TYPE_ERROR,
                '',
                'Cancel'
            );
            wp_redirect( home_url() );
            die();
        }

        $id_payment = $details->id_payment;

        $payment = new InbankPayment($id_payment);
        $current_key = $payment->debug_mode ? $payment->api_key_debug : $payment->api_key;
        if ($message->uuid !== $details->session_uuid
            || !InbankTools::verifyHmac(
                $hmac,
                $timestamp,
                stripslashes(InbankTools::getValue('message')),
                $current_key
            )
        ) {
            InbankLogs::addLog(
                $id_order,
                __('Failed to verify hmac or local session ID doesn\'t match the ID we received in cancel url.', 'woocommerce-inbank'),
                InbankLogs::TYPE_ERROR,
                '',
                'Cancel'
            );

            wp_redirect( home_url() );
            die();
        }

        $order = wc_get_order( $id_order );
        $current_order_state_wc = 'wc-' . $order->get_status();
        $new_order_status_wc = self::inbank_state_to_wc_state('cancelled');

        if ($new_order_status_wc['status'] != $current_order_state_wc) {
            try {
                $order->update_status($new_order_status_wc['status'], $new_order_status_wc['message']);
            } catch (Exception $e) {
                InbankLogs::addLog(
                    $id_order,
                    'Failed to change order status.',
                    InbankLogs::TYPE_WARNING,
                    '',
                    'Cancel',
                    $e->getMessage()
                );
            }
        }

        $details->last_state = 'cancelled';
        $details->timestamp = $timestamp;
        $details->save();

        wp_redirect( home_url() );
        die();
    }

    public function admin_options()
    {
        wp_redirect( admin_url( 'options-general.php?page=woocommerce-inbank-settings&action=edit_payment&id_payment='.str_replace('inbank_', '', $this->id) ) );
        die();
    }

    public function getOrderSum()
    {
        if (!isset($this->order_sum) || empty($this->order_sum))
            $this->order_sum = (float)$this->get_order_total();

        return $this->order_sum;
    }

    protected function get_order_total() {

        $total = 0;
        $order_id = absint( get_query_var( 'order-pay' ) );

        // Gets order total from "pay for order" page.
        if ( 0 < $order_id ) {
            $order = wc_get_order( $order_id );
            $total = (float) $order->get_total();

            // Gets order total from cart/checkout.
        } elseif ( isset(WC()->cart->total) && 0 < WC()->cart->total ) {
            $total = (float) WC()->cart->total;
        } elseif ( isset(WC()->cart->cart_contents_total) && 0 < WC()->cart->cart_contents_total ) {
            $total = (float) WC()->cart->cart_contents_total;
        } elseif ( isset(WC()->cart->subtotal) && 0 < WC()->cart->subtotal ) {
            $total = (float) WC()->cart->subtotal;
        }

        // parsing order total form WC session object
        if ($total == 0 && isset(WC()->session))
        {
            $data = WC()->session->get_session_data();
            if (!empty($data) && !empty($data['cart_totals']))
            {
                $session_totals = @unserialize($data['cart_totals']);
                if (!empty($session_totals['total']))
                    $total = (float)$session_totals['total'];
            }
        }

        return $total;
    }

    public function is_available_at_checkout()
    {
        $order_total = $this->getOrderSum();
        $cart = WC()->cart;

        // if cart is null
        if ($cart == null) {
            return false;
        }

        // if order total is zero
        if ($order_total == 0) {
            return false;
        }

        // if cart total sum is more than minimum order amount for inbank payment method
        if ($cart && 0 < $order_total && 0 < $this->inbank_payment->maximum_order_amount && $this->inbank_payment->maximum_order_amount < $order_total) {
            return false;
        }

        // if cart total sum is less than maximum order amount for inbank payment method
        if ($cart && 0 < $order_total && 0 < $this->inbank_payment->minimum_order_amount && $this->inbank_payment->minimum_order_amount > $order_total) {
            return false;
        }


        $payment = $this->inbank_payment;
        $class_name = $payment->class_name;
        $available_countries = $class_name::getAvailableCountries();
        $available_currencies = $class_name::getAvailableCurrencies();

        $country = InbankFrameworkBridge::getCountry();
        $currency = InbankFrameworkBridge::getCurrency();

        // if customer country is available by Inbank payment method
        if (!in_array(strtoupper($country), $available_countries)) {
            return false;
        }

        // if currency used is available by Inbank payment method
        if (!in_array(strtoupper($currency), $available_currencies)) {
            return false;
        }

        return true;
    }


    /**
     * @param $str
     * @param WC_Order $order
     * @return string|void
     */
    public static function custom_content_thankyou( $str, $order )
    {
        if (strpos($order->get_payment_method(), 'inbank_') !== false)
        {
            $detail = InbankDetail::getDetailsByOrderId($order->get_id());
            if (!empty($detail)) {
                try {
                    $payment = new InbankPayment($detail->id_payment);
                } catch (Exception $e) {
                    InbankLogs::addLog(
                        $order->get_id(),
                        __('Failed to load payment while trying to check for granted text.', 'woocommerce-inbank'),
                        InbankLogs::TYPE_ERROR
                    );
                    return;
                }

                $lang = strtoupper(InbankFrameworkBridge::getLocaleIso());
                if (!empty($payment->granted) && !empty($payment->granted[$lang]) && $payment->needs_approval) {
                    $options = get_option('woocommerce-inbank_global_settings');
                    $granted_state_wc = $options['order_state_granted'];
                    $current_order_state_wc = 'wc-' . $order->get_status();

                    if ($current_order_state_wc == $granted_state_wc) {
                        return $str . '<br/>' . $payment->granted[$lang];
                    }
                }
            }
        }

        return $str;
    }

    private static function inbank_state_to_wc_state($inbank_state, $options = null)
    {
        if ($options == null)
            $options = get_option( 'woocommerce-inbank_global_settings' );

        if (!isset($options['order_state_declined']))
            $options['order_state_declined'] = $options['order_state_canceled'];

        switch($inbank_state) {
            case 'cancelled':
                return ['status' => $options['order_state_canceled'], 'message' => __('The customer has cancelled the credit form filling process.', 'woocommerce-inbank')];
            case 'declined':
                return ['status' => $options['order_state_declined'], 'message' => __('Credit was declined by Inbank.', 'woocommerce-inbank')];
            case 'expired':
                return ['status' => $options['order_state_canceled'], 'message' => __('The customer did not complete Inbank form filling process by defined period of time.', 'woocommerce-inbank')];
            case 'pending':
                return ['status' => $options['order_state_pending'], 'message' => __('The customer was redirected to Inbank ePOS to fill the form.', 'woocommerce-inbank')];
            case 'granted':
                return ['status' => $options['order_state_granted'], 'message' => __('Credit has been granted to the customer by Inbank. Waiting for merchant\'s approval.', 'woocommerce-inbank')];
            case 'completed':
                return ['status' => $options['order_state_confirmed'], 'message' => __('Credit for the customer were confirmed by Inbank.', 'woocommerce-inbank')];
        }
    }

    private static function api_request_url( $request, $ssl = null, $token = null )
    {
        if ( is_null( $ssl ) ) {
            $scheme = wp_parse_url( home_url(), PHP_URL_SCHEME );
        } elseif ( $ssl ) {
            $scheme = 'https';
        } else {
            $scheme = 'http';
        }

        $args = ['wc-api' => $request];
        if (!empty($token))
            $args['token'] = $token;

        $api_request_url = add_query_arg( $args , trailingslashit( home_url( '', $scheme ) ) );

        return esc_url_raw( apply_filters( 'woocommerce_api_request_url', $api_request_url, $request, $ssl ) );
    }
}
