<?php

/**
 * The file that defines the core plugin class
 *
 * A class definition that includes attributes and functions used across both the
 * public-facing side of the site and the admin area.
 *
 * @link       https://inbank.ee
 * @since      1.0.0
 *
 * @package    Woocommerce_Inbank
 * @subpackage Woocommerce_Inbank/includes
 */

/**
 * The core plugin class.
 *
 * This is used to define internationalization, admin-specific hooks, and
 * public-facing site hooks.
 *
 * Also maintains the unique identifier of this plugin as well as the current
 * version of the plugin.
 *
 * @since      1.0.0
 * @package    Woocommerce_Inbank
 * @subpackage Woocommerce_Inbank/includes
 * @author     Inbank
 */
class Woocommerce_Inbank {

    /**
     * The loader that's responsible for maintaining and registering all hooks that power
     * the plugin.
     *
     * @since    1.0.0
     * @access   protected
     * @var      Woocommerce_Inbank_Loader    $loader    Maintains and registers all hooks for the plugin.
     */
    protected $loader;

    /**
     * The unique identifier of this plugin.
     *
     * @since    1.0.0
     * @access   protected
     * @var      string    $plugin_name    The string used to uniquely identify this plugin.
     */
    protected $plugin_name;

    /**
     * The current version of the plugin.
     *
     * @since    1.0.0
     * @access   protected
     * @var      string    $version    The current version of the plugin.
     */
    protected $version;

    /**
     * Define the core functionality of the plugin.
     *
     * Set the plugin name and the plugin version that can be used throughout the plugin.
     * Load the dependencies, define the locale, and set the hooks for the admin area and
     * the public-facing side of the site.
     *
     * @since    1.0.0
     */
    public function __construct() {
        if ( defined( 'WOOCOMMERCE_INBANK_VERSION' ) ) {
            $this->version = WOOCOMMERCE_INBANK_VERSION;
        } else {
            $this->version = '2.1.2';
        }
        $this->plugin_name = 'woocommerce-inbank';

        if (!defined('WOOCOMMERCE_INBANK_DEBUG'))
            define('WOOCOMMERCE_INBANK_DEBUG', false);

        $this->load_dependencies();
        $this->init_payment_gateway();
        $this->set_locale();
        $this->define_admin_hooks();
        $this->define_public_hooks();
    }

    /**
     * Load the required dependencies for this plugin.
     *
     * Include the following files that make up the plugin:
     *
     * - Woocommerce_Inbank_Loader. Orchestrates the hooks of the plugin.
     * - Woocommerce_Inbank_i18n. Defines internationalization functionality.
     * - Woocommerce_Inbank_Admin. Defines all hooks for the admin area.
     * - Woocommerce_Inbank_Public. Defines all hooks for the public side of the site.
     *
     * Create an instance of the loader which will be used to register the hooks
     * with WordPress.
     *
     * @since    1.0.0
     * @access   private
     */
    private function load_dependencies() {

        /**
         * Composer autoload
         */
        require_once plugin_dir_path( dirname( __FILE__ ) ) . 'vendor/autoload.php';

        /**
         * The class responsible for orchestrating the actions and filters of the
         * core plugin.
         */
        require_once plugin_dir_path( dirname( __FILE__ ) ) . 'includes/class-woocommerce-inbank-loader.php';

        /**
         * The class responsible for orchestrating updates
         * core plugin.
         */
        require_once plugin_dir_path( dirname( __FILE__ ) ) . 'includes/class-woocommerce-inbank-updater.php';

        /**
         * The class responsible for defining internationalization functionality
         * of the plugin.
         */
        require_once plugin_dir_path( dirname( __FILE__ ) ) . 'includes/class-woocommerce-inbank-i18n.php';

        /**
         * The class responsible for defining all actions that occur in the admin area.
         */
        require_once plugin_dir_path( dirname( __FILE__ ) ) . 'admin/class-woocommerce-inbank-admin.php';

        /**
         * The class responsible for defining all settings in the admin area.
         */
        require_once plugin_dir_path( dirname( __FILE__ ) ) . 'admin/class-woocommerce-inbank-settings.php';

        /**
         * The class responsible for defining all actions that occur in the public-facing
         * side of the site.
         */
        require_once plugin_dir_path( dirname( __FILE__ ) ) . 'public/class-woocommerce-inbank-public.php';

        $this->loader = new Woocommerce_Inbank_Loader();

    }

    /**
     * Define the locale for this plugin for internationalization.
     *
     * Uses the Woocommerce_Inbank_i18n class in order to set the domain and to register the hook
     * with WordPress.
     *
     * @since    1.0.0
     * @access   private
     */
    private function set_locale() {

        $plugin_i18n = new Woocommerce_Inbank_i18n();

        $this->loader->add_action( 'plugins_loaded', $plugin_i18n, 'load_plugin_textdomain' );

    }

    /**
     * Initializing payment gateway
     *
     * @since    1.0.0
     * @access   private
     */
    private function init_payment_gateway()
    {
        add_filter( 'woocommerce_payment_gateways', array($this, 'inbank_add_gateway_class') );
        add_action( 'plugins_loaded', array($this, 'inbank_load_payment_gateway') );
    }

    /**
     * Adding Inbank to payment gateways array
     *
     * @since    1.0.0
     * @access   private
     */
    public function inbank_add_gateway_class($gateways)
    {
        // protection
        if (InbankTools::check_prerequisites() !== true)
            return;

        $need_approval = false;
        $payments = InbankPayment::getAllPaymentMethods();

        // automatically updating Inbank products (one product per page call only on cart or checkout)
        InbankBalticsProduct::updatePaymentData();

        foreach ($payments as $payment) {
            $gateways[] = new WC_Inbank_Payment_Gateway($payment['id_payment']);

            if ((int)$payment['needs_approval'] == 1 && $need_approval == false)
                $need_approval = true;
        }

        if ($need_approval)
            add_filter('woocommerce_thankyou_order_received_text', array('WC_Inbank_Payment_Gateway', 'custom_content_thankyou'), 10, 2);

        return $gateways;
    }

    /**
     * Loading class responsible for payment gateway
     */
    public function inbank_load_payment_gateway()
    {
        // protection
        if (InbankTools::check_prerequisites() !== true)
            return;

        require_once plugin_dir_path( dirname( __FILE__ ) ) . 'includes/class-woocommerce-inbank-payment-gateway.php';
    }

    /**
     * Loading class responsible for payment gateway
     */
    public function check_for_updates()
    {
        global $wp_version;

        // updating plugin
        Woocommerce_Inbank_Updater::update();

        // checking for new updates to remote Inbank server
        InbankUpdater::check($this->plugin_name, $this->version, $wp_version);
    }


    /**
     * Register all of the hooks related to the admin area functionality
     * of the plugin.
     *
     * @since    1.0.0
     * @access   private
     */
    private function define_admin_hooks()
    {
        $plugin_admin = new Woocommerce_Inbank_Admin( $this->get_plugin_name(), $this->get_version() );
        $plugin_settings = new Woocommerce_Inbank_Settings( $this->get_plugin_name(), $this->get_version() );

        $this->loader->add_action( 'admin_enqueue_scripts', $plugin_admin, 'enqueue_styles' );
        $this->loader->add_action( 'admin_enqueue_scripts', $plugin_admin, 'enqueue_scripts' );
        $this->loader->add_action( 'admin_footer', $plugin_admin, 'media_selector_print_scripts' );

        $this->loader->add_filter( 'woocommerce_order_get_payment_method_title', $plugin_admin, 'fix_inbank_payment_name_in_orders_list', 999999, 2 );
        $this->loader->add_filter( 'woocommerce_gateway_title', $plugin_admin, 'fix_inbank_payment_name_in_order_page', 999999, 2 );
        $this->loader->add_action( 'wp_ajax_inbank_get_products', $plugin_admin, 'ajax_get_products' );

        $this->loader->add_action( 'admin_menu', $plugin_settings, 'setup_plugin_options_menu' );
        //$this->loader->add_action( 'admin_init', $plugin_settings, 'initialize_payment_options' );
        //$this->loader->add_action( 'admin_init', $plugin_settings, 'initialize_api_options' );
        $this->loader->add_action( 'admin_menu', $this, 'check_for_updates' );

        $this->loader->add_filter('plugin_action_links_woocommerce-inbank/woocommerce-inbank.php', $plugin_settings, 'filter_add_action_links');

        if (WOOCOMMERCE_INBANK_DEBUG && !file_exists(plugin_dir_path(dirname(__FILE__)).'debug/'))
            mkdir(plugin_dir_path(dirname(__FILE__)).'debug/');

    }

    public function woocommerce_inbank_cron_update()
    {
        $token = InbankTools::getValue('token', '');
        $hash = md5('woocommerce-inbank'.AUTH_SALT);

        if ($token == $hash) {
            InbankPayment::updateAllProducts();
            echo "Inbank payments have been successfully updated!";
        }

        die();
    }

    /**
     * Register all of the hooks related to the public-facing functionality
     * of the plugin.
     *
     * @since    1.0.0
     * @access   private
     */
    private function define_public_hooks() {

        $plugin_public = new Woocommerce_Inbank_Public( $this->get_plugin_name(), $this->get_version() );

        $this->loader->add_action( 'wp_enqueue_scripts', $plugin_public, 'enqueue_scripts' );
        $this->loader->add_action( 'wp_enqueue_scripts', $plugin_public, 'enqueue_styles' );
        $this->loader->add_action('wp_ajax_woocommerce_inbank_cron_update', $this, 'woocommerce_inbank_cron_update');
        $this->loader->add_action('wp_ajax_nopriv_woocommerce_inbank_cron_update', $this, 'woocommerce_inbank_cron_update');

        if ((function_exists('is_product') && is_product()) || !function_exists('is_product'))
            InbankPayment::addCalcHooks();
    }

    /**
     * Run the loader to execute all of the hooks with WordPress.
     *
     * @since    1.0.0
     */
    public function run() {
        $this->loader->run();
    }

    /**
     * The name of the plugin used to uniquely identify it within the context of
     * WordPress and to define internationalization functionality.
     *
     * @since     1.0.0
     * @return    string    The name of the plugin.
     */
    public function get_plugin_name() {
        return $this->plugin_name;
    }

    /**
     * The reference to the class that orchestrates the hooks with the plugin.
     *
     * @since     1.0.0
     * @return    Woocommerce_Inbank_Loader    Orchestrates the hooks of the plugin.
     */
    public function get_loader() {
        return $this->loader;
    }

    /**
     * Retrieve the version number of the plugin.
     *
     * @since     1.0.0
     * @return    string    The version number of the plugin.
     */
    public function get_version() {
        return $this->version;
    }

}
