<?php

class InbankBalticsAPI
{

    private $object;
    private $api_url;

    /**
     * InbankBalticsAPI constructor.
     * @param $object InbankPayment
     * @param $api_url string api url
     * @throws Exception
     */
    public function __construct($object, $api_url)
    {
        if (is_a($object, "InbankPayment")) {
            $this->object = $object;
        } else {
            throw new Exception('Object must be an instance of InbankPayment');
        }
        $this->api_url = $api_url;
    }

    /**
     * Performs a get request to specified endpoint with specified data
     * @param $endpoint
     * @param  array  $data
     * @return mixed
     */
    protected function get($endpoint, $data = [])
    {
        $curl = curl_init();
        $this->setDefaultOpts($curl);
        curl_setopt($curl, CURLOPT_URL, $this->getFullUrl($endpoint, $data));
        curl_setopt($curl, CURLOPT_SSL_VERIFYPEER, false);
        curl_setopt($curl, CURLOPT_SSL_VERIFYHOST, false);

        $response = curl_exec($curl);
        curl_close($curl);

        return $this->responseHandler($response);
    }

    /**
     * Performs the post request to specified endpoint
     * @param $endpoint
     * @param $data
     * @return mixed
     */
    protected function post($endpoint, $data = [])
    {
        $curl = curl_init();
        $this->setDefaultOpts($curl);
        curl_setopt($curl, CURLOPT_POST, 1);
        $jsonDataEncoded = json_encode($data);
        curl_setopt($curl, CURLOPT_POSTFIELDS, $jsonDataEncoded);
        curl_setopt($curl, CURLOPT_URL, $this->getFullUrl($endpoint));
        curl_setopt($curl, CURLOPT_SSL_VERIFYPEER, false);
        curl_setopt($curl, CURLOPT_SSL_VERIFYHOST, false);

        $response = curl_exec($curl);
        curl_close($curl);

        return $this->responseHandler($response);
    }

    /**
     * Builds the full url suitable for API call
     * @param $endpoint
     * @param  array  $data
     * @return string
     */
    protected function getFullUrl($endpoint, $data = [])
    {
        if ($this->object->debug_mode) {
            $shop_uuid = $this->object->shop_uuid_debug;
        } else {
            $shop_uuid = $this->object->shop_uuid;
        }

        return $this->api_url."shops/".$shop_uuid."/".$endpoint.(!empty($data) ? "?".http_build_query($data) : null);
    }

    /**
     * Returns correct product code depending if product is in debug mode or not
     * @return string product code
     */
    private function getProductCode()
    {
        if ($this->object->debug_mode) {
            return $this->object->product_code_debug;
        } else {
            return $this->object->product_code;
        }
    }

    /**
     * Handles the API response
     * @param $response
     * @return mixed
     */
    protected static function responseHandler($response)
    {
        $response = json_decode($response);

        return $response;
    }

    /**
     * Applies default parameters to curl
     * @param $curl
     */
    protected function setDefaultOpts(&$curl)
    {
        if ($this->object->debug_mode) {
            $api_key = $this->object->api_key_debug;
        } else {
            $api_key = $this->object->api_key;
        }
        curl_setopt($curl, CURLOPT_HTTPHEADER, [
            'Authorization: Bearer '.$api_key,
            'Content-Type: application/json',
        ]);
        curl_setopt($curl, CURLOPT_RETURNTRANSFER, 1);
        curl_setopt($curl, CURLOPT_AUTOREFERER, 1);
        curl_setopt($curl, CURLOPT_FOLLOWLOCATION, 1);
        curl_setopt($curl, CURLOPT_CONNECTTIMEOUT, 5);
        curl_setopt($curl, CURLOPT_TIMEOUT, 10);
    }

    /*
     * API Calls
     */

    /**
     * Get product info from InBank API
     */
    public function getInfo()
    {
        $endpoint = "products";

        return $this->get($endpoint, ["product_code" => $this->getProductCode()]);
    }

    /**
     * Get params used for displaying calculator in the product page
     * @param  array  $params
     * @return mixed
     */
    public function getCreditCalculation($params = [])
    {
        $endpoint = "calculations";
        if (!isset($params["response_level"])) {
            $params["response_level"] = "simple";
        }
        if (!isset($params['period'])) {
            if (!empty($this->object->max_period)) {
                $params["period"] = $this->object->max_period;
            } else {
                $periods = explode(',', $this->object->period);
                $max_period = max($periods);
                $params["period"] = $max_period;
            }
        }
        if (!isset($params['currency'])) {
            $params['currency'] = strtoupper(InbankFrameworkBridge::getCurrencySymbol());
        }
        if (!isset($params['payment_day'])) {
            $params['payment_day'] = $this->object->default_payment_day;
        }
        $params["product_code"] = $this->getProductCode();

        return $this->post($endpoint, $params);
    }

    public function initiateSession($request_body)
    {
        $endpoint = 'pos_sessions';

        return $this->post($endpoint, $request_body);
    }

    public function getSessionDetails($session_uuid)
    {
        $endpoint = 'pos_sessions/'.$session_uuid;

        return $this->get($endpoint);
    }

    public function getContractDetails($contract_uuid)
    {
        $endpoint = 'contracts/'.$contract_uuid;

        return $this->get($endpoint);
    }

    public function approveContract($contract_uuid)
    {
        $endpoint = 'contracts/'.$contract_uuid.'/merchant_approval';

        return $this->post($endpoint);
    }

    public function cancelContract($contract_uuid)
    {
        $endpoint = 'contracts/'.$contract_uuid.'/cancel';

        return $this->post($endpoint);
    }
}